package cn.iocoder.yudao.module.system.api.robot;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.tenant.core.context.TenantContextHolder;
import cn.iocoder.yudao.module.mqtt.api.common.CommonApi;
import cn.iocoder.yudao.module.system.api.robot.dto.*;
import cn.iocoder.yudao.module.system.api.robot.processor.RequestProcessor;
import cn.iocoder.yudao.module.system.api.robot.vo.RobotInformationVO;
import cn.iocoder.yudao.module.system.config.ratelimiter.SystemRateLimiter;
import cn.iocoder.yudao.module.system.constant.area.FloorAreaConstant;
import cn.iocoder.yudao.module.system.constant.path.PathPlanningTopicConstant;
import cn.iocoder.yudao.module.system.constant.robot.RobotTaskChcheConstant;
import cn.iocoder.yudao.module.system.service.robot.RobotInformationService;
import cn.iocoder.yudao.module.system.util.redis.RedisUtil;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@RestController // 提供 RESTful API 接口，给 Feign 调用
@Validated
public class RobotStatusApiImpl implements RobotStatusApi {

    @Resource
    private RedisUtil redisUtil;

    @Autowired
    private RobotInformationService robotInformationService;

    @Value("${zn.robot_position_cache_time:10}")
    private Long robotPositionCacheTime;

    @Resource
    private RequestProcessor processor;

    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;

    @Resource
    private CommonApi commonApi;

    /**
     * 更新机器人点位/异常/能否做任务
     *
     * @param robotStatusDataDTO
     * @return
     */
    @Override
    @SystemRateLimiter(time = 1, count = 150, keyArg = "robotStatusUpdate", message = "机器人上报点位超过限流")
    public void robotStatusUpdate(RobotPoseStatusDTO robotStatusDataDTO) {
        updateRobotPosed(robotStatusDataDTO);
    }

    /**
     * 更新点位信息
     *
     * @param robotStatusDataDTO
     */
    private void updateRobotPosed(RobotPoseStatusDTO robotStatusDataDTO) {
        TenantContextHolder.setTenantId(1L);
        if (ObjectUtil.isEmpty(robotStatusDataDTO) || ObjectUtil.isEmpty(robotStatusDataDTO.getMac())) {
            log.info("机器人上报的信息不全 :{}", JSON.toJSONString(robotStatusDataDTO));
            return;
        }
        String floorAreaKey = RobotTaskChcheConstant.ROBOT_FLOOR_AREA + robotStatusDataDTO.getMac();
        Object floorAreaObject = redisUtil.get(floorAreaKey);
        FloorZoneDTO floorZoneDTO = JSONUtil.toBean((String) floorAreaObject, FloorZoneDTO.class);
        String robotNo = robotInformationService.getRobotNoByMac(robotStatusDataDTO.getMac());
        String pose2dKey = RobotTaskChcheConstant.ROBOT_INFORMATION_POSE_BAT + robotStatusDataDTO.getMac();
        Object object = redisUtil.get(pose2dKey);
        RobotStatusDataPoseDTO robotStatusDataPoseDTO = JSONUtil.toBean((String) object, RobotStatusDataPoseDTO.class);

        if (ObjectUtil.isNotEmpty(robotStatusDataDTO.getPose2d())) {
            robotStatusDataPoseDTO.setX(robotStatusDataDTO.getPose2d().getX());
            robotStatusDataPoseDTO.setY(robotStatusDataDTO.getPose2d().getY());
            robotStatusDataPoseDTO.setYaw(robotStatusDataDTO.getPose2d().getYaw());
        }
        robotStatusDataPoseDTO.setRobotNo(robotNo);
        robotStatusDataPoseDTO.setFloor(floorZoneDTO.getFloor());
        robotStatusDataPoseDTO.setArea(floorZoneDTO.getArea());
        redisUtil.set(pose2dKey, JSON.toJSONString(robotStatusDataPoseDTO), robotPositionCacheTime);

        //机器人身上是否有货
        String taskStatusKey = RobotTaskChcheConstant.ROBOT_TASK_STATUS + robotStatusDataDTO.getMac();
        String cargoDetectedKey = RobotTaskChcheConstant.ROBOT_CARGO_DETECTED + robotStatusDataDTO.getMac();
        redisUtil.set(taskStatusKey, robotStatusDataDTO.getTaskStatus(), robotPositionCacheTime);
        redisUtil.set(cargoDetectedKey, robotStatusDataDTO.getCargoDetected(), robotPositionCacheTime);

        // -- 通过mac 地址获取车辆信息 - (并且加入到缓存中)
        Map<String, RobotInformationVO> robotInformationVOS = robotInformationService.getAllRobotByRedis();
        RobotInformationVO robotInformationVO = robotInformationVOS.get(robotStatusDataDTO.getMac());
        if (robotInformationVO == null) {
            robotInformationVO = robotInformationService.getRobotByRedis(robotStatusDataDTO.getMac());
        }
        robotInformationVO.setPose2d(robotStatusDataPoseDTO);

        Map<String, Object> map = new HashMap<>();
        String value = FloorAreaConstant.FLOOR_AREA_ROBOT + floorZoneDTO.getFloor() + "-" + floorZoneDTO.getArea();
        map.put(robotStatusDataDTO.getMac(), JSON.toJSONString(robotInformationVO));
        redisUtil.hmset(value, map, 20);

        // 合并请求 - 这里接受到的数据都丢给 RequestProcessor - 再整合数据通过WebSocket丢给前端
        processor.handleRequest(floorZoneDTO.getFloor() + "_" + floorZoneDTO.getArea(),
                robotStatusDataDTO.getMac(), JSONUtil.toJsonStr(robotInformationVO));
        sendToPP(robotStatusDataPoseDTO);
    }

    private void sendToPP(RobotStatusDataPoseDTO robotStatusDataPoseDTO) {
        taskExecutor.execute(() -> {
            commonApi.commonMethod(robotStatusDataPoseDTO, PathPlanningTopicConstant.AGV_POSE);
        });
    }
}
